import { Request, Response, NextFunction, Router } from 'express';
import { Magic, MAGIC_MIME_TYPE } from 'mmmagic';
import { findFile, returnError } from '../Helpers/ApiHelper';
import { convertFileToPNG } from '../Models/ConversionModel';

const magic = new Magic(MAGIC_MIME_TYPE);

const convertController = async (req: Request, res: Response, next: NextFunction) => {

    // Check if any files were sent over.
    if ( !req.files || req.files?.length <= 0 ) {
        return returnError(res, {
            "message": "No files have been provided, this api endpoint requres files to be able to process them",
            "error":   "Empty files array"
        })
    }

    // Get our file
    const inputFile = findFile('inputFile', req.files as Express.Multer.File[]);

    // Check if we have an input File
    if ( inputFile == null ) {
        return returnError(res, {
            "message": "Broken file name in request, please try again.",
            "error":   "Files are in the call, but we require the name inputFile to be sent over."
        })
    }

    let inputFileType: string | null = null;

    // Get the input file format
    await new Promise<string | string[]>((_r, _e) => { 
        magic.detectFile(inputFile.path, (err, fileType) => {
            if ( err ) {
                return _e(err);
            } else {
                return _r(fileType);
            }
        })
    })
    .then( ftype => {
        console.log(ftype);

        if ( typeof ftype != 'object' ) {
            inputFileType = ftype;
        } else {
            inputFileType = ftype[0];
        }

    })
    .catch( err => {
        console.error(err);
    })

    if ( !!!inputFileType ) {
        return returnError(res, {
            message: "Could not determine a valid input file type",
            error:   "Could not determine a valid input file type",
        })
    }

    // Check if we have the target Format
    const targetFormat = req.body.targetFormat

    // Predefine the result
    let result: string | null = null;

    // Check for the input/output formats
    if ( targetFormat == 'png' && inputFileType == 'application/pdf' ) {
        await convertFileToPNG(inputFile)
        .then( data => {
            result = data.toString(`base64`);
        })
        .catch( err => {
            console.error(err);
        })
    }

    res.status(200).send({
        message: `Conversion completed`,
        result: result,
    });

}

export { convertController }